/*
 * hash.c --
 *
 *	This file implements the hashing algorithms specified by the
 *	token service protocol.
 */

static char rcsid[] =
"$Id: hash.c,v 1.2 2001/07/03 21:46:26 gmt Exp $";
 
#include "tokInt.h"


/*  TokHash(s) -- hash a string, returning non-negative 31-bit integer  */

int
TokHash(char *s) {
    int h = 0;
    while (*s != '\0') {
	h = 37 * h + (unsigned char) *s++;
    }
    return h & 0x7FFFFFFF;
}

/*  TokRehash(n) -- return next hash value in pseudorandom sequence  */

int
TokRehash(int n) {
    /* Constants are per Knuth vol 2 [1969], pp. 155-157. */
    return (314159261 * n + 453816707) & 0x7FFFFFFF;
}

/*  TokHashList(strings) -- return non-negative hash of list of strings  */

int 
TokHashList(char *p[]) {
    char *s;
    int h = 0;
    while ((s = *p++) != NULL) {
	h = 39 * h + TokHash(s);
    }
    return h & 0x7FFFFFFF;
}
