/*  globals.h -- global variables of TopoVista  */


/*
 *  variables prefaced by "global" are both declared and defined by this
 *  file, depending on how that macro is defined.
 */

#ifndef global
  #define global extern
  #define initial(v) /**/
#endif



/* windows (GL/GLUT handles) */

global int ctlWin;		/* control window */
global int ovhWin;		/* overhead ("2D") window */
global int pspWin;		/* perspective ("3D") window */


/* terrain data */

global int tileOrder;		/* K value from initial tile */
global int rows, cols;		/* rows and cols per tile (both = 2^K+1) */

global int utmZone;		/* Universal Transverse Mercator zone */
global char initName[SHORT_TILE_NAME];	/* name of initial tile xxxUyyyc */

global int mPerCell;		/* horizontal resolution (10 or 30) */
global int tileSizeM;		/* width (and height) of tile in meters */

global Pnm **pnmArray;		/* array of pnm files to color tiles */
global char **layerName;	/* names of layers */
global int nLayers;		/* number of color layers */

global int xxmin, xxmax;	/* UTM coordinates of tiles bounding box */
global int yymin, yymax;

global int zzmin, zzmax;	/* range of elevations seen */


/* viewing options */

global int colorLayer;		/* color layer to display */

global int twoDMesh	initial(NONE2D);	/* show 2-D mesh? */
global int threeDMesh	initial(0);		/* show 3-D mesh? */

global int eyeIsRelative initial(0);		/* does eye ht follow surface?*/
global int eyeZRelative initial(20);		/* decimeters */
global int eyeZRelativeInc initial(10);		/* decimeters */
global int eyeZAbsolute initial(1000);		/* decimeters */
global int eyeZAbsoluteInc initial(1000);	/* decimeters */

/* vertical exaggeration */
global double vexagger		initial(IVEXAGGER);
global double vexaggerInc	initial(IVEXAGGERINC);

global int cullBackface	initial(0);	/* cull backfaces */
global int cullFOV	initial(0);	/* cull outside of FOV */
global int smoothSurface initial(1);	/* initially show smooth surface */

/* parameters used to judge triangle fitness */
/* Triangle err(dm)/distance(m) from eye must be < errThresh */
global double errThresh          initial(IERRTHRESH);	/* dm / m */
global double errThreshInc       initial(IERRTHRESHINC);

/* Triangles < k*distThresh meters from eye must have size < 2^k */
global double distThresh	initial(IDISTTHRESH);
global double distThreshInc	initial(IDISTTHRESHINC);


/* current viewpoint */

global Coord3f eye;	/* eye location */
global Coord3f ref;	/* reference point */

global Coord3f eyeInit;	/* initial eye location */
global Coord3f refInit;	/* initial reference point */

global int fovType;	/* shape of FOV (see defns.h) */
global Coord2f fovLLpt, fovULpt, fovLRpt, fovURpt;
global Coord2f fovTnorm, fovBLnorm, fovBRnorm, fovLnorm, fovRnorm;


/* other */

global Lock wrapLock;			/* wrap list mutual exclusion lock */
					/* also locks xxmin etc. */

global int flightStep	initial(0);	/* current step in flightpath
					 * 0 means no flight in progress
					 * -1 means current flight should stop
					 */

global FILE *flightRecord initial(NULL);	/* file to record flight */
global int flightNumber initial(0);	/* number of recording being made */

global int numSurfTri;			/* number of triangles in approx */

global int tflag	initial(0);	/* calculate timing info? */

global int redoSurface	initial(1);	/* do we need new surface approx? */
global int redoRelief	initial(1);	/* do we need to make new relief map? */

#ifdef TESTING
global int doingLoop	initial(0);	/* doing a loop around perimeter? */
global int loopRefresh	initial(0);	/* what is refreshed during loop? */
global int triPerLoop	initial(0);	/* total #tri drawn during loop */
#endif


/* defined in main.c (too complex for macros above): */

extern Color elevMap[];		/* elevation colors */
extern double z2level;		/* convert z to index into elevMap */

/* these macros should not be used elsewhere */
#undef global
#undef initial
