/*  infile.c -- I/O functions for reading from a file  */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "topo.h"

static int peekFile(inp);
static int charFile(inp);
static void closeFile(inp);

typedef struct {
  struct instruct in;
  FILE *fp;
} ins_file;



/*  openFile(filename) -- create instruct for reading from file.  */

inp openFile(char *filename)
{
  FILE *fp = fopen(filename, "rb");
  if (!fp)
    return NULL;
  SET_BINARY_MODE(fp);
  return openFP(fp);
}



/*  openFP(fp) -- create instruct for reading from file pointer.  */

inp openFP(FILE *fp)
{
  ins_file *in = alloc(sizeof(ins_file));
  in->in.peek = peekFile;
  in->in.rchar = charFile;
  in->in.rbytes = readBytes;
  in->in.rline = readLine;
  in->in.close = closeFile;
  in->fp = fp;
  return &in->in;
}



/*  peekFile(in) -- peek at next character from file.  */

static int peekFile(inp in)
{
  ins_file *inf = (ins_file *) in;
  int c = getc(inf->fp);
  if (c != EOF)
    ungetc(c, inf->fp);
  return c;
}



/*  charFile(in) -- read character from file.  */

static int charFile(inp in)
{
  ins_file *inf = (ins_file *) in;
  return getc(inf->fp);
}



/*  closeFile(in) -- close file and invalidate structure.  */

static void closeFile(inp in)
{
  fclose(((ins_file*)in)->fp);
  memset(in, 0, sizeof(ins_file));
  free(in);
}
