/*  inmem.c -- I/O functions for reading from memory */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "topo.h"

static int peekMem(inp);
static int charMem(inp);
static void closeMem(inp);

typedef struct {
  struct instruct in;		/* standard object header */
  size_t len;			/* remaining bytecount */
  unsigned char *buf;		/* data buffer */
  unsigned char *next;		/* next byte to read */
} ins_mem;



/*  openMem(addr, len) -- create instruct for reading from memory.  */

inp openMem(void *addr, size_t len)
{
  ins_mem *in = alloc(sizeof(ins_mem));
  in->in.peek = peekMem;
  in->in.rchar = charMem;
  in->in.rbytes = readBytes;
  in->in.rline = readLine;
  in->in.close = closeMem;
  in->buf = alloc(len);
  in->next = in->buf;
  in->len = len;
  memcpy(in->buf, addr, len);
  return &in->in;
}



/*  peekMem(in) -- peek at next character from memory.  */

static int peekMem(inp in)
{
  ins_mem *mem = (ins_mem *) in;
  if (mem->len == 0)
    return EOF;
  else
    return *mem->next;
}



/*  charMem(in) -- read next character from memory.  */

static int charMem(inp in)
{
  ins_mem *mem = (ins_mem *) in;
  if (mem->len == 0)
    return EOF;
  mem->len--;
  return *mem->next++;
}



/*  closeMem(in) -- close memory instruct.  */

static void closeMem(inp in)
{
  free(((ins_mem *)in)->buf);		/* free buffer */
  memset(in, 0, sizeof(ins_mem));	/* prevent object reuse after free */
  free(in);				/* free object */
}
