/*  main.c -- TopoVista startup  */

#ifdef WIN32
  #include <windows.h>
#endif

#include "topo.h"


/*  declare and initialize variables from globals.h  */
#define global
#define initial(v) = v
#include "globals.h"



/*  elevation color map, in ascending order */
/*  (also: sea level is dark blue, handled by special case in the code)  */

Color elevMap[] = {
  { 0.90, 0.86, 0.75 },
  { 0.85, 0.79, 0.62 }, /* cream */
  { 0.83, 0.793, 0.624 },
  { 0.819, 0.75, 0.455 },
  { 0.741, 0.694, 0.364 },
  { 0.721, 0.858, 0.273 },
  { 0.676, 0.858, 0.273 }, /* yellow green */
  { 0.676, 0.858, 0.273 },
  { 0.559, 0.754, 0.273 },
  { 0.429, 0.663, 0.273 },
  { 0.429, 0.663, 0.273 },
  { 0.416, 0.663, 0.283 }, /* light olive */
  { 0.446, 0.663, 0.308 },
  { 0.476, 0.663, 0.333 },
  { 0.506, 0.663, 0.358 },
  { 0.536, 0.663, 0.383 },
  { 0.566, 0.663, 0.408 },
  { 0.596, 0.663, 0.433 }, /* purplish */
  { 0.626, 0.663, 0.458 },
  { 0.656, 0.663, 0.483 },
  { 0.686, 0.663, 0.508 },
  { 0.728, 0.663, 0.533 },
  { 1.00, 1.00, 1.00 }, /* white */
  { 1.00, 1.00, 1.00 },
  { 1.00, 1.00, 1.00 },
  { 1.00, 1.00, 1.00 },
  { 1.00, 1.00, 1.00 },
  { 1.00, 1.00, 1.00 },
  { 1.00, 1.00, 1.00 },
  { 1.00, 1.00, 1.00 },
  { 1.00, 1.00, 1.00 },
  { 1.00, 1.00, 1.00 },
  { 1.00, 1.00, 1.00 },
  { 1.00, 1.00, 1.00 },
  { 1.00, 1.00, 1.00 },
  { 1.00, 1.00, 1.00 },
  { 1.00, 1.00, 1.00 },
  { 1.00, 1.00, 1.00 },
  { 1.00, 1.00, 1.00 },
  { 1.00, 1.00, 1.00 },
  { 1.00, 1.00, 1.00 },
  { 1.00, 1.00, 1.00 },
  { 1.00, 1.00, 1.00 },
  { 1.00, 1.00, 1.00 },
  { 1.00, 1.00, 1.00 },
};

double z2level = sizeof(elevMap) / sizeof(elevMap[0]) / (float)USHRT_MAX;


static char *bsname(char *f);
static int options(int argc, char *argv[]);
static int initConfig(char *filename);
static void usage(char *progname);


#ifdef TSLOOKUP
static char *tsvcCfg = NULL;	/* token service configuration file name */
#endif


/*  main  */

int main(int argc, char **argv)
{
  Pnm *p;
  int opti, i, n;
  char *path, *s;
  char cwd[1000];

  getcwd(cwd, sizeof(cwd));		/* remember working directory */
  glutInit(&argc, argv);		/* initialize GLUT system */
  chdir(cwd);				/* restore working directory */
			/* (glutInit changes working dir in MacOSX 10.2.6) */

  INITLOCK(wrapLock);			/* initialize wrap list mutex */

  opti = options(argc, argv);		/* process command-line options */
  if (opti >= argc)			/* if no files then error */
    usage(argv[0]);

  if( !initConfig(argv[opti]) ) {
    fprintf(stderr,
       "Tile name %s is not of the form .../xxxUyyyc.tvg\n", argv[opti]);
    exit(1);
  }

  #ifdef TSLOOKUP
    if (tsvcCfg!=NULL)	/* if TSVC cfg provided, initialize TSVC handle  */
      initTSVCHandle(tsvcCfg);
  #endif

  /* ======== read color layers and paths ======== */
  nLayers = 0;
  if( opti + 1 < argc ) {
    layerName = (char **)malloc((argc-opti)*sizeof(char *));
    pnmArray = (Pnm **)malloc((argc-opti)*sizeof(Pnm *));
    if( layerName == NULL || pnmArray == NULL ) nospace();
  }
  for( i = opti + 1; i < argc; i++ ) {
    s = argv[i];
    n = strlen(s);
    if( tileID(s) && strcmp(s + n - 4, ".tvg") == 0 ) {
      /* ignore additional tile names given as command args */
    } else if( (p = readPNM(s)) ) {
      pnmArray[nLayers] = p;
      layerName[nLayers] = bsname(s);
      ++nLayers;
    } else if( readFlightPath(s) ) {
      /* do nothing */
    } else if( readPaths(s) ) {
      /* do nothing */
    } else {
      fprintf(stderr, "Cannot read %s\n", s);
    }
  }
  colorLayer = nLayers-1;

  /* add first tile, except if dealing with an asynchronous protocol */
  path = argv[opti];
  if( !preskip(path, "tvtp:") && !preskip(path, "tvrp:")) {
    if( !getTile(argv[opti]) ) {
      fprintf(stderr, "Can't read initial tile %s.\n", argv[opti]);
      exit(1);
    }
  }
  eyeZAbsolute = zzmax + 20;
  setEyeRefHeight();

  /* open windows */
  openCtlWin();			/* control window */
  openOvhWin();			/* overhead (2-D) window */
  openPspWin();			/* perspective (3-D) window */

  /* process events */
  glutMainLoop();

  /*NOTREACHED*/
  return 0;
}



static char *bsname(char *f)
{
  char *s, *e, *r;
  int i;

  if ((s = strrchr(f, '/')) == NULL) {
    if ((s = strrchr(f, '\\')) == NULL)
      s = f;
    else
      s += 1;
  }
  else {
    s += 1;
  }

  if ((e = strchr(s,'.')) == NULL)
    e = s+strlen(s);

  r = (char *)malloc((e-s+1)*sizeof(char));
  if( r == NULL ) nospace();
  i = 0;
  while(s<e)
    r[i++] = *s++;
  r[i] = 0;

  return r;
}



/*  options(argc, argv) -- interpret command-line arguments  */

static int options(int argc, char *argv[])
{
  register int i;
  register char *s;

  for (i = 1; i < argc; ++i)
    {
      s = argv[i];
      if (*s != '-') break;
      switch (*++s)
	{
#ifdef TSLOOKUP
        case 'c':
          if (*++s)
            tsvcCfg = salloc(s);
          else if (++i < argc)
            tsvcCfg = salloc(argv[i]);
          else
            usage(argv[0]);
          break;
#endif
	case 'd':
	  if (*++s)
	    distThresh = atof(s);
	  else if (++i < argc)
	    distThresh = atof(argv[i]);
	  else
	    usage(argv[0]);
	  break;
	case 'e':
	  if (*++s)
	    errThresh = atof(s) / 100.0;
	  else if (++i < argc)
	    errThresh = atof(argv[i]) / 100.0;
	  else
	    usage(argv[0]);
	  break;
	case 't':
	  tflag = 1;
	  break;
	case 'v':
	  if (*++s)
	    vexagger = atof(s);
	  else if (++i < argc)
	    vexagger = atof(argv[i]);
	  else
	    usage(argv[0]);
	  break;
	default:
	  usage(argv[0]);
	}
    }
  return(i);

}



/*  initConfig(filename) -- deduce configuration information from tile name  */

static int initConfig(char *filename) {
  int spacing;
  Tile t;

  if( !setID(&t, filename) )
    return 0;

  tileOrder = TIDORDER(t.tid);		/* tile order ("k" value) */
  utmZone = TIDZONE(t.tid);		/* UTM projection zone */
  spacing = 1 << tileOrder;		/* spacing of tiles in gridpoints */
  rows = cols = spacing + 1;		/* tile dimensions (seams overlap) */

  if( TIDYYY(t.tid) * spacing > 206737 ) /* coordinate threshold */
    mPerCell = 10;			/* must be 10m grid above 18.67 N */
  else
    mPerCell = 30;			/* must be 30m grid below 56 N */

  tileSizeM = spacing * mPerCell;	/* tile size in meters */
  copyPrefix(filename);
  strcpy(initName, t.tname);

  xxmin = XXX2UTM(TIDXXX(t.tid));	/* set x/y min/max based on init tile */
  yymin = YYY2UTM(TIDYYY(t.tid));
  xxmax = xxmin + tileSizeM;
  yymax = yymin + tileSizeM;

  eye[0] = eyeInit[0] = xxmin;
  eye[1] = eyeInit[1] = yymin;
  ref[0] = refInit[0] = xxmin + 0.5 * tileSizeM;
  ref[1] = refInit[1] = yymin + 0.5 * tileSizeM;
  return 1;
}



/*  usage(progname) -- issue usage message, and abort  */

static void usage(char *progname)
{
  fprintf(stderr,
    "usage: %s [options] tile [coloring...] [track...] [flightpath...]\n",
    progname);
  fprintf(stderr,
    "  -d nn.nn   distance threshold (m)  (default=%.1f)\n", IDISTTHRESH);
  fprintf(stderr,
    "  -e nn.nn   error threshold (m/km)  (default=%.1f)\n", 100 * IERRTHRESH);
  fprintf(stderr,
    "  -v nn.nn   vertical scaling factor (default=%.1f)\n", IVEXAGGER);
  fprintf(stderr,
    "  -t         time the hierarchy and surface creation\n");
  #ifdef TSLOOKUP
    fprintf(stderr,
      "  -c <file>  token service config; tile denotes starting tile\n");
  #endif
  fprintf(stderr,
    "  tile: filename or URL (HTTP:, TVRP:, or TVTP:) of form xxxUyyyc.tvg\n");
  exit(1);
}
