/*  misc.c -- miscellaneous functions  */

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include "topo.h"



/*
 * Holds the prefix and suffix values used to form tile names.
 * The prefix is something like "tucson/" or "http://www.abc.com/TV4data/".
 * The suffix is typically ".tvg".
 */
static char prefix[MAX_TILE_NAME];
static char suffix[MAX_TILE_NAME];



/*
 * Sets the prefix and suffix variables for later use.
 * The prefix is the substring up through the last occurrence of '/' in path.
 * The suffix is the first "." after the prefix, and everything that follows.
 */
void copyPrefix(const char *arg)
{
  int n;
  const char *pre, *suf; 

  pre = strrchr(arg, '/');
  if( pre == NULL ) {
    strcpy(prefix, "");
    pre = arg;
  } else {
    n = pre - arg + 1;
    strncpy(prefix, arg, n);
    prefix[n] = '\0';
  }

  suf = strchr(pre, '.');
  if( suf == NULL ) {
    strcpy(suffix, "");
  } else {
    strcpy(suffix, suf);
  }
}



/*
 * Generates the xxxUyyyc name of a tile.
 */
char *synthName(char *buf, int zone, int xxx, int yyy, int k)
{
  sprintf(buf, "%d%c%d%c", xxx, 'A' + zone - 1, yyy, 'a' + k - 1);
  return buf;
}



/*
 * Generates the full path of a tile.
 */
char *synthPath(char *buf, int zone, int xxx, int yyy, int k)
{
  char name[SHORT_TILE_NAME];
  synthName(name, zone, xxx, yyy, k);
  sprintf(buf, "%s%s%s", prefix, name, suffix);
  return buf;
}
