/*  normal.c -- get the normal of surface  */

#include "topo.h"

static void normalDFS(Triangle t,
		      int x1, int y1, int x2, int y2, int x3, int y3);
static void triNormal(Triangle t,
		      int x1, int y1, int x2, int y2, int x3, int y3);



void surfaceNormal(Tile *tile)
{
  Triangle t;

  tile->normal = zalloc(3 * rows * cols * sizeof(float));
  t.tile = tile;
  t.i = LCHILD(0);
  normalDFS(t, tile->lx1,tile->ly1,tile->lx2,tile->ly2,tile->lx3,tile->ly3);
  t.i = RCHILD(0);
  normalDFS(t, tile->rx1,tile->ry1,tile->rx2,tile->ry2,tile->rx3,tile->ry3);
}



static void normalDFS(Triangle t,
		      int x1, int y1, int x2, int y2, int x3, int y3)
{
  int xm, ym;

  if( ISLEAF(t) ) {
    triNormal(t, x1, y1, x2, y2, x3, y3);
  } else {
    xm = (x1 + x2) / 2;
    ym = (y1 + y2) / 2;
    t.i = LCHILD(t.i);
    normalDFS(t, x3, y3, x1, y1, xm, ym);
    t.i += 1;
    normalDFS(t, x2, y2, x3, y3, xm, ym);
  }
}



static void triNormal(Triangle t,
		      int x1, int y1, int x2, int y2, int x3, int y3)
{
  float z1, z2, z3;
  float nx, ny, nz;

  z1=EMDATA(t.tile, x1, y1);
  z2=EMDATA(t.tile, x2, y2);
  z3=EMDATA(t.tile, x3, y3);

  GET_NORMAL;
  EMNORMAL(t.tile,x1,y1,0)+=nx;
  EMNORMAL(t.tile,x1,y1,1)+=ny;
  EMNORMAL(t.tile,x1,y1,2)+=nz;

  EMNORMAL(t.tile,x2,y2,0)+=nx;
  EMNORMAL(t.tile,x2,y2,1)+=ny;
  EMNORMAL(t.tile,x2,y2,2)+=nz;

  EMNORMAL(t.tile,x3,y3,0)+=nx;
  EMNORMAL(t.tile,x3,y3,1)+=ny;
  EMNORMAL(t.tile,x3,y3,2)+=nz;
}
