/*  protos.h -- function prototypes for TopoVista  */

/* color.c */
void initColors(void);

/* control.c */
void openCtlWin(void);
void keyCB(unsigned char key, int x, int y);
void specialCB(int key, int x, int y);

/* decode.c */
int buildTile(Tile *tile);

/* emit.c */
void emitSurface(int type);

/* eps.c */
void outputEPS(int doSort, char *filename);

/* errcalc.c */
int errDFS(Triangle t, int unknown, int hgt, int o, 
	  int x1, int y1, int x2, int y2, int x3, int y3);

/* fetch.c */
void initTSVCHandle(char *cfg);
Wrap *fetch(char *name, int type);
void *destroy(Wrap *w);

/* infov.c */
void getVerts(Triangle t,
	      int *px1, int *py1, int *px2, int *py2, int *px3, int *py3);
int triInFOV(Triangle t);

/* interest.c */
Tile *mostNeeded(void);
IntList *interests(void);

/* input.c */
/* (see input.h) */

/* lock.c */
void initlock(Lock *k, char *where);
void lock(Lock *k, char *where);
void rattle(Lock *k, char *where);
void unlock(Lock *k, char *where);

/* main.c */
int main(int argc, char **argv);

/* mqueue.c */
MsgQueue *messageQueue(void);
void sendMessage(MsgQueue *mq, void *msg);
void* rcvMessage(MsgQueue *mq);

/* neighbor.c */
Triangle nbr1(Triangle t);
Triangle nbr2(Triangle t);
Triangle nbr3(Triangle t);

/* misc.c */
void copyPrefix(const char *arg);
char *synthName(char *buf, int zone, int xxx, int yyy, int k);
char *synthPath(char *buf, int zone, int xxx, int yyy, int k);

/* normal.c */
void surfaceNormal(Tile *);

/* overhead.c */
void openOvhWin(void);
void ovhDisplayCB(void);
void ovhReshapeCB(int w, int h);
void mouseto(int btn, int x, int y);
void followPathCB(int dummy);
void ovhHome( void );
void ovhEnd( void );
void ovhVis(int x, int y);
int inOvh(int x, int y);
int ovhCounter(void);

/* path.c */
void emitPaths( void );
Path* readPaths(char* filename);
FlightPath* readFlightPath(char* filename);

/* pnmops.c */
/* (see pnmops.h) */

/* print.c */
void printSurface(char *fname);

/* split.c */
void split(Triangle t);

/* surface.c */
void errSurface(void);

/* tile.c */
Tile *alcTile(Wrap *wrap);
int setID(Tile *tile, char *url);
void freeTile(Tile *tile);
int addRec(Tile *tile, void *buf, int len);
int getTile(char *name);
int reqTileXY(int xxx, int yyy);
Wrap *wantTile(char *name);
void colorTile(Tile *tile);
Tile *tileCovering( double xx, double yy );

/* tvrp.c */
void rpfetch(char *name);

/* tvtp.c */
void tvfetch(char *name);

/* trans.c */
/* utils.c */
/* (see basic.h) */

/* view.c */
int zval(Tile *tile, int x, int y);
void openPspWin(void);
void buildSurface(void);
void renderPsp(void);
void setEyeRefHeight(void);
void postRedisplays( int refresh );

/* wrap.c */
Wrap *newWrap(char *name, int type);
Wrap *addWrap(Wrap *w);
Wrap *findWrap(char *name, int type);
void *delWrap(Wrap *w);
WrapGen *genWraps(int type);
Wrap *nextWrap(WrapGen *g);
void endGen(WrapGen *g);
