/*  split.c -- triangle splitting operations  */

#include "topo.h"



/* Triangle t is split into lt and rt.  Neighbors 1 & 2 of lt and
 * rt are known to be the same size as lt and rt.  Neighbor 3 of
 * lt is neighbor 2 of t (n3lt == n2t).  If n2t is smaller than
 * t then n3lt is same size as lt, o.w. n3lt is bigger than lt (and
 * n1n2t == lt is smaller than n2t == n3lt).
 * Similar gibberish holds for rt.
 */

static void kidsOnSurf(Triangle t)
    /* Push the surface down to the children of t.  t must not be a leaf. */
{
  Triangle lt, rt, n2t, n1t;

  if( !ONSURFACE(t) ) {
    fprintf(stderr, "kids triangle %p %d\n", t.tile, t.i);
    exit(1);
  }
  assert(ONSURFACE(t));
  assert(!ISLEAF(t));

  INFO(t) &= ~SURFACE;
  lt.tile = t.tile;
  lt.i = LCHILD(t.i);
  PUTONSURFACE(lt);
  rt.tile = t.tile;
  rt.i = RCHILD(t.i);
  PUTONSURFACE(rt);

  n2t = nbr2(t);
  if( n2t.tile != NULL ) {
    if( SMALLN2(t) ) {
      INFO(n2t) &= ~BIGGER3;
    } else {
      INFO(n2t) |= SMALLER1;
      INFO(lt) |= BIGGER3;
    }
  }
  n1t = nbr1(t);
  if( n1t.tile != NULL ) {
    if( SMALLN1(t) ) {
      INFO(n1t) &= ~BIGGER3;
    } else {
      INFO(n1t) |= SMALLER2;
      INFO(rt) |= BIGGER3;
    }
  }
  if( INFO(t) & ALLINFOV ) {
    INFO(lt) |= ALLINFOV;
    INFO(rt) |= ALLINFOV;
  } else if( INFO(t) & PARTINFOV ) {
    INFO(lt) |= triInFOV(lt);
    INFO(rt) |= triInFOV(rt);
  }
  numSurfTri += 1;
}



void split(Triangle t)
{
  Triangle n3t;

  if( !ONSURFACE(t) ) {
    fprintf(stderr, "split triangle %p %d\n", t.tile, t.i);
    exit(1);
  }

  assert(ONSURFACE(t));
  n3t = nbr3(t);
  if( n3t.tile != NULL && BIGN3(t) ) {
    split(n3t);				/* propagate split */
    if( t.i & 1 ) {
      n3t.i = RCHILD(n3t.i);
    } else {
      n3t.i = LCHILD(n3t.i);
    }
  }
  if( n3t.tile != NULL ) {		/* t is not LEAF so n3t is not LEAF */
    kidsOnSurf(n3t);
  }
  kidsOnSurf(t);
}
