//  tTimer.java -- tests the Timer class
//
//  When run with any argument, suppresses clock portion of output
//  for greater reproducibility of output (for automated testing).
//  May give irreproducible results on busy systems.

//  Some portions of the test below have been disabled because
//  they no longer pass reliably (although they once did).


import topovista.toksvc.*;

class tTimer {


public static void main(final String args[]) throws Exception {

    final Queue q = new Queue();

    Timer t1 = new Timer() {
	public void alarm(Timer t, Object val) { q.put("t1." + val); }
    };
    Timer t2 = new Timer() {
	public void alarm(Timer t, Object val) { q.put("t2." + val); }
    };
    Timer t3 = new Timer() {
	public void alarm(Timer t, Object val) { q.put("t3." + val); }
    };
    Timer t4 = new Timer() {
	public void alarm(Timer t, Object val) { q.put("t4." + val); }
    };
    Timer t5 = new Timer() {
	public void alarm(Timer t, Object val) { q.put("t5." + val); }
    };

    // start consumer thread
    Thread c = new Thread() {
	private long tbase = System.currentTimeMillis();
	private long tprev = tbase;
	public void run() {
	    for (;;) {
		Object val = q.get();
		long t = System.currentTimeMillis();
		if (args.length == 0) {
		    System.out.print((t - tbase) + "\t" + (t - tprev) + "\t");
		}
		System.out.println(val);
		tprev = t;
	    }
	}
    };
    c.setDaemon(true);
    c.start();

    q.put("----- b");		// test near-immediate calls
    t4.set(0, "b1");
    Thread.sleep(10);
    t4.set(-1, "b2");
    Thread.sleep(10);
    t4.set(-47, "b3");
    Thread.sleep(10);

    q.put("----- c");		// test multi timers fire in correct order
    t5.set(50, "c1");
    t4.set(40, "c2");
    t3.set(30, "c3");
    t2.set(20, "c4");
    t1.set(10, "c5");
    Thread.sleep(100);

    q.put("----- d");		// retest multi timers
    t5.set(50, "d1");
    t4.set(40, "d2");
    t3.set(30, "d3");
    t2.set(20, "d4");
    t1.set(10, "d5");
    Thread.sleep(100);

    q.put("----- e");		// retest multi timers
    t5.set(50, "e1");
    t4.set(40, "e2");
    t3.set(30, "e3");
    t2.set(20, "e4");
    t1.set(10, "e5");
    Thread.sleep(100);

    q.put("----- f");		// test cancellation before start
    t1.set(10, "f1");
    t2.set(30, "f2");
//    t3.set(20, "f3"); // provokes infinite loop on IBM linux JDK 130-20010502
    t1.clear();
    t2.clear();
    t3.clear();
    Thread.sleep(50);

    q.put("----- g");		// test cancellation  after start
    t1.set(90, "g1");
    t2.set(80, "g2");
    t3.set(70, "g3");
    t4.set(60, "g4");
    t5.set(50, "g5");
    Thread.sleep(10);
    t1.clear();
    t2.clear();
    t3.clear();
    t4.clear();
    t5.clear();
    Thread.sleep(100);

    q.put("----- h");		// test revision of interval
    t5.set(90, "h1");
    t5.set(70, "h2");
    t5.set(50, "h3");
    t5.set(30, "h4");
    t5.set(10, "h5");
    Thread.sleep(50);

    q.put("----- i");		// retest multi timers
    t5.set(50, "i1");
    t4.set(40, "i2");
    t3.set(30, "i3");
    t2.set(20, "i4");
    t1.set(10, "i5");
    Thread.sleep(100);

    q.put("----- zzz");
    Thread.sleep(100);		// catch stragglers
    System.out.println("----- exiting");
}


} // class tTimer
