package topovista.explorer;

import java.io.*;
import java.util.*;

import topovista.grid.*;



/**
 *  A World object maintains a global collection of known grid cells.
 */

class World {

    private Hashtable registry = new Hashtable();	// cell inventory



/**
 *  Constructs an empty World object.
 */
public World() {
}



/**
 *  Constructs a World object and initializes with "hasdata" cells
 *  based on the contents of a directory.
 */
public World(File dir) throws IOException {
    FilenameFilter ff = new FilenameFilter() {
	public boolean accept(File d, String name) {
	    return name.endsWith(Survey.TVG_EXTENSION);
	}
    };
    String[] names = dir.list(ff);
    if (names == null) {
	throw new IOException("not a directory: " + dir);
    }
    for (int i = 0; i < names.length; i++) {
	String s = names[i];
	s = s.substring(0, s.length() - 4);
	find(s).hasdata = true;
    }
}



/**
 *  Returns a cell from the world, creating it if necessary.
 */
public synchronized Cell find(String name) {
    Cell ce = (Cell) registry.get(name);
    if (ce == null) {
	ce = new Cell(name);
	registry.put(name, ce);
    }
    return ce;
}



/**
 *  Returns a cell from the world, creating it if necessary.
 */
public Cell find(int zone, int x, int y, int k) {
    return find(TileGrid.makeName(zone, x, y, k));
}



/**
 *  Returns a randomly chosen cell.
 */
public synchronized Cell choose() {

    int n = registry.size();
    if (n == 0) {
	return null;
    }

    int k = (int) (n * Math.random());

    Enumeration e = registry.elements();
    while (k-- > 0) {
	e.nextElement();
    }
    return (Cell) e.nextElement();
}



/**
 *  Returns a list of the four cells neighboring the given cell,
 *  beginning with the cell to the east and moving counterclockwise.
 */
public synchronized Cell[] neighbors(Cell ce) {

    int zone = ce.zone;
    int x = ce.x;
    int y = ce.y;
    int k = ce.k;
    Cell[] list = new Cell[4];

    list[0] = find(zone, x + 1, y, k);
    list[1] = find(zone, x, y + 1, k);
    list[2] = find(zone, x - 1, y, k);
    list[3] = find(zone, x, y - 1, k);
    return list;
}



/**
 *  Returns a list of all known cells.
 */
public synchronized Cell[] all() {

    Cell[] list = new Cell[registry.size()];

    Enumeration e = registry.elements();
    int i = 0;
    while (e.hasMoreElements()) {
	list[i++] = (Cell) e.nextElement();
    }

    return list;
}



/**
 *  Returns a string representation of this World.
 */
public String toString() {
    return "World(" + registry.size() + ")";
}



} // class World
