package topovista.rpserver;

import java.util.*;



/**
 *  The cleaner runs periodically in the background and removes
 *  requests that have grown stale, either through aging or by
 *  having been superseded.
 *
 *  The cleaner is a Thread object that must be explicitly
 *  started by the code that creates it.
 */

class Cleaner extends Thread implements TVRP {


    private LinkedList[] rqqueues;	// incoming request queues to monitor



/**
 *  Constructs a cleaner to watch the given set of request queues.
 */
Cleaner(LinkedList[] rqqueues) {
    this.rqqueues = rqqueues;	// remember the queue array
}



/**
 *  Loops forever, running the cleaning process.
 */
public void run() {

    for (;;) try {
	try {
	    Thread.sleep(CLEANER_MSEC);
	} catch (Exception unimportant) {
	}
	long cutoff = System.currentTimeMillis() - 1000 * CLIENT_DEAD_SEC;
	synchronized (rqqueues) {
	    for (int i = 0; i < rqqueues.length; i++) {
		Iterator it = rqqueues[i].iterator();
		while (it.hasNext()) {
		    Request r = (Request) it.next();
		    Message m = r.msg;
		    Client c = m.client;
		    if (m != c.latest || m.timestamp < cutoff) {
			it.remove();
		    }
		}
	    }
	}
    } catch (Throwable e) {
	e.printStackTrace();
	System.exit(1);
    }
}



/**
 *  Returns a string representation of this cleaner.
 */
public String toString() {
    return "Cleaner(" + rqqueues + ")";
}



} // class Cleaner
