//  CatalogMessage.java -- transmits client holdings to a server


package topovista.toksvc;

import java.net.*;



/**
 *  A CatalogMessage transmits a record of client
 *  token holdings to a server.
 */
class CatalogMessage extends Message {

    Tk[] tokens;		// array of tokens



final static int MTYPE = 13;	// type code for this Message class



/**
 *  Constructor for synthesizing a new message.
 */
CatalogMessage(Tk[] tokens) {
    super(MTYPE);
    this.tokens = tokens;
}



/**
 *  Adds the encoding of this object to a Pickle.
 */
Pickle pickle(Pickle p) {
    super.pickle(p);
    p.put(tokens.length);
    for (int i = 0; i < tokens.length; i++) {
	tokens[i].pickle(p);
    }
    return p;
}



/**
 *  Constructs a CatalogMessage by reading from a Pickle.
 *  The tokens in the message are created as ServerTokens.
 */
CatalogMessage(Pickle p) {
    super(p);
    int n = p.getInt();
    tokens = new Tk[n];
    for (int i = 0; i < n; i++) {
	tokens[i] = new ServerToken(p);
    }
}



/**
 *  Returns a string representation of this message.
 */
public String toString() {
    return super.toString() + "[" + tokens.length + "]";
}



}  // class CatalogMessage
