//  DataMessage.java -- a structured form of a datagram.


package topovista.toksvc;

import java.net.*;



/**
 *  A DataMessage sends an array of arbitrary bytes.
 */
class DataMessage extends Message {

    byte[] data;	// data buffer


final static int MTYPE = 1;	// type code for this Message class



/**
 *  Adds the encoding of this object to a Pickle.
 */
Pickle pickle(Pickle p) {
    super.pickle(p);
    p.put(data);
    return p;
}



/**
 *  Constructs a DataMessage by reading from a Pickle.
 */
DataMessage(Pickle p) {
    super(p);
    data = p.getByteArray();
}



/**
 *  Constructor for synthesizing a new message.
 */
DataMessage(byte[] data) {
    super(MTYPE);
    this.data = data;
}



/**
 *  Replaces the data field.
 */
void setData(byte[] data) {
    this.data = data;
}



/**
 *  Returns a string representation of this message.
 */
public String toString() {
    return super.toString() + "[" + data.length + "]";
}



}  // class DataMessage
