//  RequestMessage.java -- requesting assignment of a token


package topovista.toksvc;

import java.net.*;



/**
 *  A RequestMessage asks for the assignment of a token.
 */
class RequestMessage extends NumberedMessage {

    int access;				// access type: +1 shared, -1 exclusive



final static int MTYPE = 21;	// type code for this message class



/**
 *  Constructor for synthesizing a new message.
 */
RequestMessage(long msgnum, Tk token, int access) {
    super(MTYPE, msgnum, token);
    this.access = access;
}



/**
 *  Adds the encoding of this object to a Pickle.
 */
Pickle pickle(Pickle p) {
    super.pickle(p);
    p.put(access);
    return p;
}



/**
 *  Constructs a RequestMessage by reading from a Pickle.
 */
RequestMessage(Pickle p) {
    super(p);
    access = p.getInt();
}



/**
 *  Returns a string representation of this message.
 */
public String toString() {
    String s;
    switch (access) {
	case Token.SHARED:	s = "(s)";  break;
	case Token.EXCLUSIVE:	s = "(x)";  break;
	default:		s = "(?),";  break;
    }
    return super.toString() + s;
}



}  // class RequestMessage
