//  ReturnMessage.java -- update data and/or deassign token


package topovista.toksvc;

import java.net.*;



/**
 *  A ReturnMessage updates token data,
 *  deassigns a token, or both.
 */
class ReturnMessage extends NumberedMessage {

    int flags;			// action flags



final static int MTYPE = 24;	// type code for this message class



/**
 *  Designates a data update for a token.
 */
final static int UPDATE = 1;



/**
 *  Designates a deassignment of a token.
 */
final static int DEASSIGN = 2;



/**
 *  Constructor for synthesizing a new message.
 */
ReturnMessage(long msgnum, Tk token, int flags) {
    super(MTYPE, msgnum, token);
    this.flags = flags;
}



/**
 *  Adds the encoding of this object to a Pickle.
 */
Pickle pickle(Pickle p) {
    super.pickle(p);
    p.put(flags);
    return p;
}



/**
 *  Constructs a ReturnMessage by reading from a Pickle.
 *  The constructed token is a ServerToken.
 */
ReturnMessage(Pickle p) {
    super(p);
    flags = p.getInt();
}



/**
 *  Returns a string representation of this message.
 */
public String toString() {
    String s1 = ((flags & DEASSIGN) != 0) ? "d" : "";
    String s2 = ((flags & UPDATE) != 0) ? "u" : "";
    String s3 = ((flags & ~(DEASSIGN + UPDATE)) != 0) ? "?" : "";

    return super.toString() + "(" + s1 + s2 + s3 + ")";
}



}  // class ReturnMessage
