//  RosterMessage.java -- distribute roster of servers and clients


package topovista.toksvc;

import java.net.*;



/**
 *  A RosterMessage distributes the configuration of active
 *  servers and clients.
 *  As a subclass of LeadMessage, it also serves as the leader's heartbeat.
 */
class RosterMessage extends LeadMessage {

    byte[] sstates;		// server state array
    SessionList slist;		// client session list



final static int MTYPE = 5;	// type code for this Message class



/**
 *  Constructor for synthesizing a new message.
 */
RosterMessage(byte[] sstates, SessionList slist) {
    super(MTYPE);
    this.sstates = sstates;
    this.slist = slist;
}



/**
 *  Returns a string representation of this message.
 */
public String toString() {
    return super.toString() +
	'(' + slist.size() + ',' + ConfigMessage.simage(sstates) + ')';
}



/**
 *  Adds the encoding of this object to a Pickle.
 */
Pickle pickle(Pickle p) {
    super.pickle(p);
    p.put(sstates);
    slist.pickle(p);
    return p;
}



/**
 *  Constructs a RosterMessage by reading from a Pickle.
 */
RosterMessage(Pickle p) {
    super(p);
    sstates = p.getByteArray();
    slist = new SessionList(p);
}



}  // class RosterMessage
