//  SessionList.java -- maintains a list of client sessions


package topovista.toksvc;

import java.util.*;



/**
 *  A SessionList maintains a list of client Sessions.
 */
class SessionList {

    private Hashtable h;



/**
 *  Constructs a new, empty SessionList.
 */
SessionList() {
    h = new Hashtable();
}



/**
 *  Constructs a SessionList by reading a Pickle.
 */
SessionList(Pickle p) {
    this();
    int n = p.getInt();
    for (int i = 0; i < n; i++) {
	add(new Session(p));
    }
}



/**
 *  Adds the encoding of this SessionList to a Pickle.
 */
synchronized Pickle pickle(Pickle p) {
    p.put(h.size());
    for (Enumeration e = h.elements(); e.hasMoreElements(); ) {
	((Session)e.nextElement()).pickle(p);
    }
    return p;
}



/**
 *  Adds a session to the list.
 */
synchronized void add(Session s) {
    h.put(new Integer(s.getID()), s);
}



/**
 *  Finds the session with the given ID, if present.
 */
synchronized Session get(int sessID) {
    return (Session)h.get(new Integer(sessID));
}



/**
 *  Finds a session in the list, if present.
 */
synchronized Session get(Session s) {
    return (Session)h.get(new Integer(s.getID()));
}



/**
 *  Removes a session from the list.
 */
synchronized void remove(Session s) {
    h.remove(new Integer(s.getID()));
}



/**
 *  Returns the number of entries in the session list.
 */
int size() {
    return h.size();
}


/**
 *  Returns an enumeration of the sessions in the list.
 */
Enumeration elements() {
    return h.elements();
}



/**
 *  Returns a string representation of this message.
 */
public String toString() {
    return "SessionList(" + h.size() + ")";
}



}  // class SessionList
