package topovista.tvsucker;

import java.util.*;



/**
 *  Navigates randomly around the starting tile.
 */

class RandomNav extends Navigator {

    private Tile center;	// center tile
    private double radius;	// radius for selection
    private int nleft;		// number of tiles to go

    private Hashtable visited = new Hashtable();  // tiles already visited



public RandomNav(String firstTile, int ntiles) throws Exception {
    center = new Tile(firstTile);
    radius = 2 + Math.sqrt(ntiles / Math.PI);
    nleft = ntiles;
}



/*
 *  Returns a randomly chosen tile that has not been returned before
 *  and is near the center tile.  Returns null after returning the
 *  specified number of tiles.
 */
public synchronized Tile nextTile() throws Exception {

    if (--nleft < 0) {
	return null;
    }
    for (;;) {
	double angle = 2 * Math.PI * Math.random();
	double r = radius * Math.random();
	int dx = (int) (r * Math.cos(angle) - 0.5);
	int dy = (int) (r * Math.sin(angle) - 0.5);
	Tile t = new Tile(center.zone, center.xxx + dx, center.yyy + dy,
	    center.k, center.extn);
	if (visited.put(t.getName(), t) == null) {
	    return t;
	}
    }
}



/*
 *  Returns a string representation of this navigator.
 */
public String toString() {
    return "RandomNav(" + center.getName() + "," + nleft + ")";
}



} // class RandomNav
