package topovista.tvsucker;

/**
 *  Navigates in a spiral around the starting tile.
 */

class SpiralNav extends Navigator {

    private Tile start;		// initial tile
    private Tile next;		// next tile to return

    private int tleft;		// total number left to return

    private int dir;		// current direction (0=W, 1=N, 2=E, 3=S)
    private int side;		// length of this side
    private int sleft;		// tiles left on this side



/**
 *  Constructs a spiral navigator.
 */
public SpiralNav(String firstTile, int ntiles) throws Exception {
    start = next = new Tile(firstTile);
    tleft = ntiles;
    dir = 1;
    sleft = 1;
    side = 1;
}



/**
 *  Returns tiles in a spiral around the starting point,
 *  up to the specified limit.
 */
public synchronized Tile nextTile() throws Exception {

    if (--tleft < 0) {
	return null;
    }

    Tile ret = next;
    switch (dir) {
	case 0:
	    next = new Tile(ret.zone, ret.xxx - 1, ret.yyy, ret.k, ret.extn);
	    break;
	case 1:
	    next = new Tile(ret.zone, ret.xxx, ret.yyy + 1, ret.k, ret.extn);
	    break;
	case 2:
	    next = new Tile(ret.zone, ret.xxx + 1, ret.yyy, ret.k, ret.extn);
	    break;
	case 3:
	    next = new Tile(ret.zone, ret.xxx, ret.yyy - 1, ret.k, ret.extn);
	    break;
    }

    if (--sleft == 0) {			// if time to turn
	dir = (dir + 1) % 4;
	side += dir & 1;
	sleft = side;
    }

    return ret;
}



/*
 *  Returns a string representation of this navigator.
 */
public String toString() {
    return "SpiralNav(" + start.getName() + "," + tleft + ")";
}



} // class SpiralNav
