package topovista.util;

import java.io.*;
import javax.xml.parsers.*;
import org.xml.sax.*;



/**
 *  Implements a simpleminded utility for converting GPX files to flat ASCII.
 */
public class GPXFlat extends org.xml.sax.helpers.DefaultHandler {

    PrintStream out = new PrintStream(new BufferedOutputStream(System.out));

    String data;	// data accumulator

    String time;	// saved time value
    String name;	// saved name value
    String ele;		// saved ele value
    String lat;		// saved lat value
    String lon;		// saved lon value



/**
 *  Converts GPX data to flat ASCII format.
 *  GPX is an XML data format for GPS tracks, routes, and waypoints.
 *  For more information about GPX, see www.topografix.com/gpx.asp.
 *
 *  <P> Usage:  java GPXFlat [file...]
 */
public static void main(String args[]) throws Exception {
    SAXParserFactory f = SAXParserFactory.newInstance();
    GPXFlat g = new GPXFlat();
    if (args.length == 0) {
	f.newSAXParser().parse(System.in, g);
    } else {
	for (int i = 0; i < args.length; i++) {
	    f.newSAXParser().parse(args[i], g);
	}
    }
    g.out.flush();		// ensure that buffered output is flushed
}



/**
 *  Called at the start of an XML element.
 */
public void startElement(String uri,String lname,String qname,Attributes atts)
throws SAXException {

    data = "";
    if (qname.equals("trkpt") || qname.equals("rtept") || qname.equals("wpt")) {
	time = name = ele = lat = lon = "-";
    } else if (qname.equals("rte") || qname.equals("trkseg")) {
	out.println();
    }
    if (atts != null) {
	for (int i = 0; i < atts.getLength(); i++) {
	    String aname = atts.getQName(i);
	    String value = atts.getValue(i);
	    if (aname.equals("lat")) {
		lat = value;
	    } else if (aname.equals("lon")) {
		lon = value;
	    }
	}
    }
}



/**
 *  Called at the end of an XML element.
 */
public void endElement(String uri, String sname, String qname)
throws SAXException {

    if (qname.equals("wpt")) {
	out.println();
	out.println("WPT \"" + name + "\" " + ele + " " + lat + " " + lon);
    } else if (qname.equals("trkpt")) {
	out.println("TRK " + time + " " + ele + " " + lat + " " + lon);
    } else if (qname.equals("rtept")) {
	out.println("RTE \"" + name + "\" " + ele + " " + lat + " " + lon);
    } else if (qname.equals("time")) {
	time = data;
    } else if (qname.equals("name")) {
	name = data;
    } else if (qname.equals("ele")) {
	ele = data;
    }
}



/**
 *  Called to accumulate data belonging to an element.
 */
public void characters(char[] buf, int off, int len) throws SAXException {
    data += new String(buf, off, len);
}



} // class GPXFlat
