package topovista.util;

import java.io.*;
import java.util.*;


/**
 *  Tests the Options code.  When run with arguments, parses them using
 *  a predefined options specification.  When run with no arguments,
 *  executes a canned test set.
 *
 *  @author Gregg Townsend, University of Arizona
 *  @version February, 2001
 */

class OptionsTester {


public static void main(String args[]) {

    String cmdname = "java OptionsTester";
    String[] speclist = new String[] {
	"-d",		"delete afterward",
	"-f ifile",	"read input from ifile",
	"-n nproc 1",	"use N parallel processes",	// 1 is default
	"-p",		"preserve dates and times",
	"-v",		"provide verbose commentary",
	"-x nnn",	"horizontal position",
	"-y nnn",	"vertical position",
	"src",		"data source",
	"...",		"",
	"dst",		"destination",
    };

    Options o = new Options(speclist, cmdname, args);
    if (args.length > 0) {
	dump(o, args);
    } else {
	o.setHeader("MyProgram Deluxe version 1.3.2 (A-43F-q7)");
	o.setFooter("So long, and thanks for all the fish.");
	o.usage(System.out, cmdname);
	test(o, "");
	test(o, "a b c");
	test(o, "- a b c");
	test(o, "-- -a -b -c");
	test(o, "-d -n 12 -v");
	test(o, "-d -n12 -v -- qrst");
	test(o, "-dn12 -v - mnop");
	test(o, "-dpv -f somefile -x111 -y222 -vn2 src1 src2 src3 destn");
	test(o, "-d -v -p -!");
	test(o, "-d -p -v -n");
    }
}



private static void test(Options o, String cmdline) {
    System.out.println();
    StringTokenizer tk = new StringTokenizer(cmdline);
    int n = tk.countTokens();
    String[] arglist = new String[n];
    for (int i = 0; i < n; i++) {
	arglist[i] = tk.nextToken();
    }
    try {
	o.parse(arglist);
	dump(o, arglist);
    } catch (Exception e) {
	System.out.println(cmdline);
	System.out.println(e);
    }
}



private static void dump(Options o, String[] arglist) {
    System.out.print("[i=" + o.getIndex() + "]:");
    for (int i = 0; i < arglist.length; i++) {
	System.out.print(" " + arglist[i]);
    }
    System.out.println();
    String s = o.getWhich();
    for (int i = 0; i < s.length(); i++) {
	char c = s.charAt(i);
	System.out.println("  -" + c + " " + o.getValue(c));
    }
    System.out.println("[ -f " + o.getValue('f') + " ]");
    System.out.println("[ -n " + o.getValue('n') + " ]");
}



} // class OptionsTester
